@@ -25,29 +25,71 @@ OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 """
 
+import argparse
 import uuid
 import random
 import string
 import sys
+import os
 import ast
 import re
 import inspect
+import subprocess
 
 sys.path.append('.')
 import utils
 
-
 class Scriber(object):
+    def __init__(self):
+        pass
+
+    def p(self, obj):
+        pass
+
+    def watch(self, obj):
+        pass
+
+    def iterscribe(self, obj):
+        pass
+
+    def d(self, obj, unit="-"):
+        pass
+
+    def save_logs(self, save):
+        pass
+
+
+class Watcher(object):
+    def __init__(self):
+        self.watching = []
+        self.watch_lines = {}
+
+    def watch_var(self, var):
+        self.watching.append(var)
+
+    def num_watched(self):
+        return len(self.watching)
+
+    def set_lines(self, var, lines):
+        self.watch_lines[var] = lines
+
+    def new_line_nums(self):
+        return sum(self.watch_lines.values(), [])
+
+    def vars_and_lines(self):
+        return self.watch_lines.items()
+
+
+class Runner(object):
     def __init__(self):
         self.show_line_num = True
         self.save_logs = True
+        self.initialized = False
         # p for print, d for distinguish
         self.api_calls = ['p', 'watch', 'iterscribe', 'd', 'Scriber']
         self.imports = ['re', 'pprint']
         self.desugared_lines = []
-        # TODO: Maybe modularize into separate Watcher class?
-        self.watching = []  # List of variable ids watching
-        self.watch_lines = {}  # Map of variable id to list of lines changed
+        self.watcher = Watcher()
 
     def gen_line_mapping(self, program_file):
         """Return a dictionary of lines as keys and
@@ -86,7 +128,6 @@ class Scriber(object):
         f = open(program_file, 'r')
         ast_output = ast.parse(f.read())
         f.close()
-        # print ast.dump(ast_output)
         return ast_output
 
     def write_imports(self, f):
@@ -102,31 +143,39 @@ class Scriber(object):
         self.write_imports(desugared_copy)
         program = open(program_file, 'r')
         first_call_indentation = ""
-        closing_line_num = 0
+        # Check if .close() has been added due to indentation decrease.
+        # If not, add it at the end.
+        closing_line_added = False
 
         for line_num, line_content in enumerate(program.readlines()):
             indentation = utils.get_indentation(line_content)
-            if (first_call_indentation != "" and
+            if (not closing_line_added and
+                    self.save_logs and
+                    self.initialized and
+                    first_call_indentation != "" and
+                    len(line_content) > 2 and
                     len(indentation) < len(first_call_indentation)):
-
                 closing_line = (first_call_indentation +
                                 "pyscribe_log.close()\n")
+                closing_line_added = True
                 self.desugared_lines.append(closing_line)
             if "Scriber()" in line_content:  # Line matches initial call
                 if self.save_logs:
+                    self.initialized = True
                     desugared_line = (indentation +
                                      "pyscribe_log = open('pyscribe_logs.txt', 'w')\n")
                     self.desugared_lines.append(desugared_line)
                     first_call_indentation = indentation
+                self.desugared_lines.append(line_content)
             elif line_content in line_mapping.values():  # Line matches an API call
                 self.desugared_lines.append(self.desugar_line(line_content[:-1],
                                             line_num,
                                             program_file,
                                             program_ast))  # don't want to include \n
-            elif (len(self.watching) > 0 and  # Line matches watched variable change
-                  line_num in sum(self.watch_lines.values(), [])):
+            elif (self.watcher.num_watched() > 0 and  # Line matches watched variable change
+                  line_num in self.watcher.new_line_nums()):
                 self.desugared_lines.append(line_content)
-                for var, lines in self.watch_lines.items():
+                for var, lines in self.watcher.vars_and_lines():
                     if line_num in lines:
                         self.desugared_lines.append(self.variable_change(var,
                                                                          line_num,
@@ -134,7 +183,7 @@ class Scriber(object):
                         break  # Should only be true once
             else:
                 self.desugared_lines.append(line_content)
-        if first_call_indentation == "":
+        if not closing_line_added and self.save_logs and self.initialized:
             self.desugared_lines.append("pyscribe_log.close()\n")
         program.close()
         for line in self.desugared_lines:
@@ -206,9 +255,10 @@ class Scriber(object):
     def iterscribe(self, line, line_num, indentation, program_ast):
         variable_id, variable_type = utils.get_id_and_type(line, program_ast)
         for node in ast.walk(program_ast):
+            # TODO: handle nested for loops
             if ('iter' in node._fields and
                 ast.dump(ast.parse(line).body[0]) in ast.dump(node)):
-                self.desugared_lines.insert(node.lineno-1,
+                self.desugared_lines.insert(node.lineno,
                                             self.iter_start(node,
                                                             line,
                                                             line_num,
@@ -250,10 +300,10 @@ class Scriber(object):
 
     def watch(self, line, line_num, program_file, program_ast):
         variable_id, variable_type = utils.get_id_and_type(line, program_ast)
-        self.watching.append(variable_id)
+        self.watcher.watch_var(variable_id)
         lines = filter(lambda x: x > line_num,
                        utils.lines_variable_changed(variable_id, program_file))
-        self.watch_lines[variable_id] = lines
+        self.watcher.set_lines(variable_id, lines)
         return ("Watching variable " +
                 variable_id +
                 ", currently ' + " +
@@ -261,20 +311,44 @@ class Scriber(object):
                 " + ' ' + str(" +
                 variable_id + ")")
 
+def python_file_type(string):
+    if not string.endswith(".py"):
+        msg = "%r is not a Python file" % string
+        raise argparse.ArgumentTypeError(msg)
+    return string
+
+def process_args():
+    parser = argparse.ArgumentParser(description='Let PyScribe make print debugging easier and more efficient.')
+    parser.add_argument('python_file', metavar='f', type=python_file_type, nargs='+',
+                        help='The Python file with PyScribe API calls.')
+    parser.add_argument('-r', '--run', action='store_true',
+                        help='Run the desugared version')
+    parser.add_argument('-d', '--desugared', action='store_true',
+                        help='Produce a desugared version of the file with all API calls replaced with valid Python.')
+    parser.add_argument('-c', '--clean', action='store_true',
+                        help='Produce a clean version of the file with all references to PyScribe removed')
+    parser.add_argument('-e', '--extraargs', nargs="+",
+                        help='Arguments intended to be passed to Python file when run. Must be called with --run set')
+    return parser.parse_args()
 
 def main():
-    scribe = Scriber()
-    if len(sys.argv) != 2:
-        raise KeyError("Please pass in only one python file as the single argument")
-    program_file = sys.argv[1]
-    if ".py" != program_file[-3:]:
-        raise KeyError("Please pass in a Python file as argument")
+    args = process_args()
+    scribe = Runner()
+    program_file = args.python_file[0]
     line_mapping = scribe.gen_line_mapping(program_file)
-    clean_copy = scribe.gen_clean_copy(program_file, line_mapping)
     program_ast = scribe.gen_ast(program_file)
     desugared_copy = scribe.gen_desugared(line_mapping,
                                           program_file,
                                           program_ast)
+    if args.clean:
+        scribe.gen_clean_copy(program_file, line_mapping)
+    if args.run:
+        if args.extraargs:
+            subprocess.call(['python', desugared_copy] + [arg for arg in args.extraargs])
+        else:
+            subprocess.call(['python', desugared_copy])
+    if not args.desugared:
+        os.remove(desugared_copy)
 
 if __name__ == "__main__":
     main()
